import os
 
from aws_cdk import (
    core,
    aws_codepipeline as _codepipeline,
    aws_codepipeline_actions as _codepipeline_actions,
    aws_codecommit as _codecommit,
    aws_codebuild as _codebuild,
    pipelines as _pipelines
)

from rba_cdk import (
  rba_ecr,
  rba_codepipeline_actions
)

def get_image_tag():
  if 'COMMIT_ID' in os.environ:
    return os.environ['COMMIT_ID'][:7]
  else:
    return 'latest'

class DockerPipelineStack(core.Stack):
  '''
  stack include an AWS codepipeline-based pipeline that can build docker images and deploy cdk applications.
  this stack class is opinionated.  it creates following construct based on the CodeCommit repository name:
    ECR - create an ECR that has the same name as code repo by default
    Docker Build Stage - add a stage for CodeBuild that is capable of building docker images by default.  
  '''  
  def __init__(self, scope: core.Construct, id: str, code_repo_name: str, **kwargs):
    super().__init__(scope, id, **kwargs)

    # create ecr repository for docker images
    ecr_repo = rba_ecr.ECR(self, code_repo_name)

    source_artifact = _codepipeline.Artifact()
    cloud_assembly_artifact = _codepipeline.Artifact()
    codecommit_repo = _codecommit.Repository.from_repository_name(self, 'repo', code_repo_name)

    pipeline = _pipelines.CdkPipeline(self, f'{code_repo_name}-pipeline',
      cloud_assembly_artifact=cloud_assembly_artifact,
      pipeline_name=f'{code_repo_name}-codepipeline',

      source_action=_codepipeline_actions.CodeCommitSourceAction(
        action_name='codeCommit',
        output=source_artifact,
        repository=codecommit_repo,
        variables_namespace='SourceVariables',
        code_build_clone_output=True),

      synth_action=_pipelines.SimpleSynthAction(
        source_artifact=source_artifact,
        cloud_assembly_artifact=cloud_assembly_artifact,
        install_commands=['npm install -g aws-cdk && pip install -r requirements.txt'],
        environment_variables={
          'COMMIT_ID': _codebuild.BuildEnvironmentVariable(value='#{SourceVariables.CommitId}')
        },
        synth_command='cdk synth'))
    
    docker_build_stage = pipeline.add_stage('DockerBuild')
    # create an action to build the docker image
    docker_build_action = rba_codepipeline_actions.DockerBuildAction(self,
      action_name='DockerBuild',
      input=source_artifact,
      variables_namespace='BuildVariables',
      environment_variables={
        "ECR_REPO_URI": _codebuild.BuildEnvironmentVariable(value=ecr_repo.repository_uri)
      } 
    )
    docker_build_stage.add_actions(docker_build_action)

    self.pipeline = pipeline
